package gregapi.worldgen;

import static gregapi.data.CS.F;
import static gregapi.data.CS.NB;
import static gregapi.data.CS.T;
import gregapi.block.IBlockExtendedMetaData;
import gregapi.block.IBlockPlacable;
import gregapi.code.ItemStackContainer;
import gregapi.data.CS.BlocksGT;
import gregapi.old.GT_Worldgen_Ore;

import java.util.Collection;
import java.util.Random;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

/**
 * @author Gregorius Techneticies
 */
public class Worldgen_Stone extends GT_Worldgen_Ore {
	public Worldgen_Stone(String aName, boolean aDefault, Block aBlock, int aBlockMeta, int aDimensionType, int aAmount, int aSize, int aProbability, int aMinY, int aMaxY, Collection<String> aBiomeList, boolean aAllowToGenerateinVoid) {
		super(aName, aDefault, aBlock, aBlockMeta, aDimensionType, aAmount, aSize, aProbability, aMinY, aMaxY, aBiomeList, aAllowToGenerateinVoid);
	}
	
	@Override
	public boolean executeWorldgen(World aWorld, Random aRandom, String aBiome, int aDimensionType, int aChunkX, int aChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		if (isGenerationAllowed(aWorld, aDimensionType, mDimensionType) && (mBiomeList.isEmpty() || mBiomeList.contains(aBiome)) && (mProbability <= 1 || aRandom.nextInt(mProbability) == 0)) {
			for (int i = 0; i < mAmount; i++) {
				int tX = aChunkX + aRandom.nextInt(16), tY = mMinY + aRandom.nextInt(mMaxY - mMinY), tZ = aChunkZ + aRandom.nextInt(16);
				if (mAllowToGenerateinVoid || !aWorld.getBlock(tX, tY, tZ).isAir(aWorld, tX, tY, tZ)) {
					float var6 = aRandom.nextFloat() * (float)Math.PI;
					double var7 = ((tX + 8) + MathHelper.sin(var6) * mSize / 8.0F);
					double var9 = ((tX + 8) - MathHelper.sin(var6) * mSize / 8.0F);
					double var11 = ((tZ + 8) + MathHelper.cos(var6) * mSize / 8.0F);
					double var13 = ((tZ + 8) - MathHelper.cos(var6) * mSize / 8.0F);
					double var15 = (tY + aRandom.nextInt(3) - 2);
					double var17 = (tY + aRandom.nextInt(3) - 2);
					
					for (int var19 = 0; var19 <= mSize; ++var19) {
						double var20 = var7 + (var9 - var7) * var19 / mSize;
						double var22 = var15 + (var17 - var15) * var19 / mSize;
						double var24 = var11 + (var13 - var11) * var19 / mSize;
						double var26 = aRandom.nextDouble() * mSize / 16.0D;
						double var28 = (MathHelper.sin(var19 * (float)Math.PI / mSize) + 1.0F) * var26 + 1.0D;
						double var30 = (MathHelper.sin(var19 * (float)Math.PI / mSize) + 1.0F) * var26 + 1.0D;
						int tMinX = MathHelper.floor_double(var20 - var28 / 2);
						int tMinY = Math.max(0, MathHelper.floor_double(var22 - var30 / 2));
						int tMinZ = MathHelper.floor_double(var24 - var28 / 2);
						int tMaxX = MathHelper.floor_double(var20 + var28 / 2);
						int tMaxY = Math.min(aWorld.getHeight(), MathHelper.floor_double(var22 + var30 / 2));
						int tMaxZ = MathHelper.floor_double(var24 + var28 / 2);
						
						for (int eX = tMinX; eX <= tMaxX; ++eX) {
							double var39 = (eX + 0.5D - var20) / (var28 / 2.0D);
							if (var39 * var39 < 1.0D) {
								for (int eY = tMinY; eY <= tMaxY; ++eY) {
									double var42 = (eY + 0.5D - var22) / (var30 / 2.0D);
									if (var39 * var39 + var42 * var42 < 1.0D) {
										for (int eZ = tMinZ; eZ <= tMaxZ; ++eZ) {
											double var45 = (eZ + 0.5D - var24) / (var28 / 2.0D);
											if (var39 * var39 + var42 * var42 + var45 * var45 < 1.0D) {
												Block tTargetedBlock = aWorld.getBlock(eX, eY, eZ);
												if (tTargetedBlock == NB || tTargetedBlock.isAir(aWorld, eX, eY, eZ)) {
													if (mAllowToGenerateinVoid) aWorld.setBlock(eX, eY, eZ, mBlock, mBlockMeta, 0);
												} else if (tTargetedBlock instanceof IBlockExtendedMetaData) {
													overrideBlock((IBlockExtendedMetaData)tTargetedBlock, aWorld, eX, eY, eZ);
												} else {
													if (tTargetedBlock.isReplaceableOreGen(aWorld, eX, eY, eZ, Blocks.stone) || tTargetedBlock.isReplaceableOreGen(aWorld, eX, eY, eZ, Blocks.end_stone) || tTargetedBlock.isReplaceableOreGen(aWorld, eX, eY, eZ, Blocks.netherrack)) {
														aWorld.setBlock(eX, eY, eZ, mBlock, mBlockMeta, 0);
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
			return T;
		}
		return F;
	}
	
	private void overrideBlock(IBlockExtendedMetaData aBlock, World aWorld, int aX, int aY, int aZ) {
		short aID = aBlock.getExtendedMetaData(aWorld, aX, aY, aZ);
		IBlockPlacable tBlock = null;
		if (BlocksGT.stoneToNormalOres.values().contains(aBlock)) {
			tBlock = BlocksGT.stoneToNormalOres.get(new ItemStackContainer(mBlock, 1, mBlockMeta));
		} else if (BlocksGT.stoneToSmallOres.values().contains(aBlock)) {
			tBlock = BlocksGT.stoneToSmallOres.get(new ItemStackContainer(mBlock, 1, mBlockMeta));
		} else if (BlocksGT.stoneToBrokenOres.values().contains(aBlock)) {
			tBlock = BlocksGT.stoneToBrokenOres.get(new ItemStackContainer(mBlock, 1, mBlockMeta));
		}
		if (tBlock != null) tBlock.placeBlock(aWorld, aX, aY, aZ, (byte)6, aID, null, false, true);
	}
}